#!/usr/bin/env python
#------------------------------------------------------------------
#- File: plot.py
#- Description: Make plots
#  Created: 01-Jun-2013 INFN SOS 2013, Vietri sul Mare, Italy, HBP
#------------------------------------------------------------------
import os, sys
from ROOT import *
from histutil import *
from time import sleep
from array import array
#------------------------------------------------------------------
def readAndFill(filename, treename, h):
    # open ntuple (see histutil.py for implementation)
    ntuple = Ntuple(filename, treename)
    
    # loop over ntuple
    count = 0
    for rownumber, event in enumerate(ntuple):
        if event.f_pt4l < 0: continue
        count += 1
        h.Fill(event.f_Z1mass, event.f_Z2mass, event.f_weight)
        if count % 5000 == 0:
		print count
    h.Scale(1.0/h.Integral())
#------------------------------------------------------------------
#------------------------------------------------------------------
def main():
	print "="*80
	
	setStyle()
        
	XNBINS= 30
	XMIN  =  0
	XMAX  =150
	
	YNBINS= 30
	YMIN  =  0
	YMAX  =150
        
        option = 'p'
        msize = 0.15
        nx = 2
        ny = 1
        pixels = 400
        xpos = 0.45
        ypos = 0.85
        tsize= 0.05
        treename = "HZZ4LeptonsAnalysis"
        sfilename= 'sig_gg-H-ZZ-2e2mu_8TeV_126GeV.root'
        bfilename= 'bkg_ZZ-2e2mu_8TeV.root'
        
        gROOT.ProcessLine('.L skde.cpp+')
        sKDE = skde()
        
        gROOT.ProcessLine('.L bkde.cpp+')
        bKDE = bkde()
	# ---------------------------------------------------------
	# make 2-D surface plot
	# ---------------------------------------------------------
	# set up a standard graphics style
	setStyle()

        # fill signal histogram
	hsig = mkhist2('hsig', "m_{Z1} (GeV)", "m_{Z2}",
                       XNBINS, XMIN, XMAX,
                       YNBINS, YMIN, YMAX)
        hsig.SetMarkerSize(msize)
        hsig.SetMarkerColor(kCyan+1)
	readAndFill(sfilename, treename, hsig)

        # fill background histogram
	hbkg = mkhist2('hbkg', "m_{Z1} (GeV)", "m_{Z2}",
                       XNBINS, XMIN, XMAX,
                       YNBINS, YMIN, YMAX)
        hbkg.SetMarkerSize(msize)
        hbkg.SetMarkerColor(kMagenta+1)
	readAndFill(bfilename, treename, hbkg)

        # p(x|s)
	hs = mkhist2("hs", "m_{Z1} (GeV)", "m_{Z2}",
		     XNBINS, XMIN, XMAX, YNBINS, YMIN, YMAX)
	hs.SetMinimum(0)

        # p(x|b)
	hb = mkhist2("hb", "m_{Z1} (GeV)", "m_{Z2}",
		     XNBINS, XMIN, XMAX, YNBINS, YMIN, YMAX)
	hb.SetMinimum(0)

        # compute KDE over a grid of points
	point = vector('double')(2)
	xstep = (XMAX-XMIN)/XNBINS
	ystep = (YMAX-YMIN)/YNBINS
	for i in xrange(XNBINS):
		x = XMIN + (i+0.5)*xstep
                point[0] = x
		for j in xrange(YNBINS):
			y  = YMIN + (j+0.5)*ystep
                        point[1] = y
			ps = sKDE(point)
                        pb = bKDE(point)
			hs.Fill(x, y, ps)
                        hb.Fill(x, y, pb)

        # plot surface
        
	c  = TCanvas("fig_higgszz", "", 10, 10, nx*pixels, ny*pixels)
	# divide canvas canvas along x-axis
	c.Divide(nx, ny)

	c.cd(1)
        hsig.DrawNormalized(option)
        hs.DrawNormalized('cont1 same')
        s1 = Scribe(xpos, ypos, tsize)
        s1.write('pp #rightarrow H #rightarrow ZZ #rightarrow 4l')
        c.Update()

        c.cd(2)
        hbkg.DrawNormalized(option)
        hb.DrawNormalized('cont1 same')
        s2 = Scribe(xpos, ypos, tsize)
        s2.write('pp #rightarrow ZZ #rightarrow 4l')
        c.Update()
        
        c.SaveAs(".gif")
        c.SaveAs(".pdf")
        gApplication.Run()
        sleep(10)
#----------------------------------------------------------------------
main()
